#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "gfx.h"

typedef struct { char *label; int code; int x,y; } btn_t;

#define BUT_W 70
#define BUT_H 40
#define PAD 10

static int lab1,lab2,lab3,lab4;

static btn_t buttons[]=
{
  {"RWY", 'Y'}, {"NEG",0},  {"=",0},  {"<", 0},  {"C", 0},
  {"DEG", 'G'}, {"+",  0},  {"7",0},  {"8", 0},  {"9", 0},
  {"MRC", '?'}, {"-",  0},  {"4",0},  {"5", 0},  {"6", 0},
  {"M+",  'P'}, {"*",  0},  {"1",0},  {"2", 0},  {"3", 0},
  {"M-",  'M'}, {"/",  0},  {"0",0},  {".", 0},  {":", 0},
};


static void screen()
{
  int code;
  int fh1,fh3;
  int r,c,x,y,i;
  unsigned int scr_w,scr_h;
  int winw,winh,winx,winy,
      LABW,          // Labels area
      lab1w,lab1h,lab1x,lab1y, // Main area
      lab2w,lab2h,lab2x,lab2y, // Status
      lab3w,lab3h,lab3x,lab3y, // RWY indicator
      lab4w,lab4h,lab4x,lab4y; // M indicator

  gfx_geometry(&scr_w,&scr_h);
  
  gfx_text_size(1, "9", NULL, &fh1);
  gfx_text_size(3, "9", NULL, &fh3);

  winw= 5*BUT_W + 6*PAD;


  LABW= winw - 2*PAD;

  int Mw;

  gfx_text_size(3,"M", &Mw, NULL);

  y= PAD;

  lab4w= Mw + PAD;
  lab4h= fh3+PAD;
  lab4x= PAD;
  lab4y= y;


  lab1w= LABW-lab4w;
  lab1h= lab4h;
  lab1y= y;
  lab1x= lab4x+lab4w;

  y+= lab1h + PAD;
  lab3y= y; 
  lab3h= fh1 + PAD;
  lab3w= LABW / 3;
  lab3x= PAD;

  lab2y= y;
  lab2x= lab3x+lab3w+PAD;
  lab2w= LABW-lab3w-PAD;
  lab2h= lab3h;

  y+= lab2h + PAD;
  
  for(r=0;r<5;r++)
  {
    x= PAD;
    for(c=0;c<5;c++)
    {
      buttons[c+r*5].x= x;
      buttons[c+r*5].y= y;
      x+= BUT_W + PAD;
    }
    y+= BUT_H + PAD;
  }

  winh= y;

  winx= (scr_w - winw)/2; if (winx<0) winx= 0;
  winy= (scr_h - winh)/2; if (winy<0) winy= 0;

  gfx_fill_rect(0, 0,0,scr_w,scr_h,0x00);
  gfx_fill_rect(0, winx,winy,winw,winh, 0xededed);
  gfx_draw_rect(0, winx-1,winy-1,winw+2,winh+2, 0x888888);
  lab1= gfx_label(GFX_JUST_RIGHT,3,0x00,0xcccccc, lab1x+winx,lab1y+winy,
                  lab1w,lab1h, "0");
  lab2= gfx_label(GFX_JUST_CENTER,1, 0xffffff, 0x333333,lab2x+winx,lab2y+winy,
                  lab2w,lab2h, "");
  lab3= gfx_label(GFX_JUST_CENTER,1, 0xffffff, 0x333333,lab3x+winx,lab3y+winy,
                  lab3w,lab3h, "RWY: 000");
  lab4= gfx_label(GFX_JUST_CENTER,3, 0x00, 0xcccccc,lab4x+winx,lab4y+winy,
                  lab4w,lab4h, "");
  for(i=0;i<25;i++)
  {
    code= buttons[i].code;
    if (!code) code= buttons[i].label[0];
    gfx_button(0,buttons[i].x+winx,buttons[i].y+winy,BUT_W,BUT_H,
                 buttons[i].label, GFXEV_LAST+code);
  }
  gfx_update(0,0,scr_w,scr_h);

  printf("winw= %d winh= %d\n",winw,winh);
}

static char NUM2[200], NUM1[200], MEM[200];
static int rwy;


static void printnum()
{
  gfx_label_set_text(GFX_UPDATE,lab1,NUM2);
}

static void putnum(int c)
{
  int L;
  L= strlen(NUM2);
  NUM2[L]= c; NUM2[L+1]= 0;
}

static void copy_number(int op)
{
  char buf[500];
  strcpy(NUM1, NUM2);

  sprintf(buf,"%s %c", NUM2, op);
  gfx_label_set_text(GFX_UPDATE,lab2,buf);
}

static double timval(char *str)
{
  long K,L;
  char *end;
  K= strtol(str,&end,10);
  L= strtol(end+1,NULL,10);

  if (str[0]=='-') L= -L;
  return (K*60+L);
}

static void val2tim(double V, char *R)
{
  int Z;
  Z= trunc(V);
  
  if (Z<0) 
    sprintf(R,"-%d:%02d", -Z/60, -Z%60);
  else
    sprintf(R,"%d:%02d", Z/60, Z%60);
}

static void do_op(int op)
{
  double V1, V2,R;
  if (strchr(NUM1,':'))
    V1= timval(NUM1);
  else
    V1= strtod(NUM1,NULL);

  if (strchr(NUM2,':'))
    V2= timval(NUM2);
  else
    V2= strtod(NUM2,NULL);

  R= 0.0;
  switch(op)
  {
  case '+': R= V1+ V2; break;
  case '-': R= V1- V2; break;
  case '*': R= V1* V2; break;
  case '/': if (V2!=0.0) R= V1/V2; break;
  }

  if (strchr(NUM1,':') || strchr(NUM2,':'))
   val2tim(R, NUM1);
  else
   sprintf(NUM1,"%g",R);
  strcpy(NUM2,NUM1);
  
  gfx_label_set_text(GFX_UPDATE,lab1,NUM2);
  gfx_label_set_text(GFX_UPDATE,lab2,"");
}

static void do_wind()
{
  long dir, spd;
  char *end;
  int H,X; char *HT,*LR;
  char buf[200];
  dir= strtol(NUM2, &end, 10);
  spd= strtol(end+1, NULL, 10);

  dir -= rwy;
  while(dir<0) dir+= 360;
  while(dir>=360) dir -= 360;
  if (dir>=180) { dir= 360-dir; LR= "Left"; } else LR= "Right";
  if (dir>=90)  { dir= 180-dir; HT= "Tail"; } else HT= "Head";
  H= lround(cos(dir/180.0*M_PI)*spd);
  X= lround(sin(dir/180.0*M_PI)*spd);
  sprintf(buf,"%s: %d %s: %d", HT, H, LR, X);
  gfx_label_set_text(GFX_UPDATE, lab1, buf);
  NUM2[0]= 0;
}

static void killnum() { NUM2[0]= 0; }

static int strhas(char *hay,char *nee)
{
  int i;
  for(i=0;nee[i];i++) if (strchr(hay,nee[i])) return 1;
  return 0;
}

void android_main(void *state,size_t statelen)
{
  gfx_event_t event;
  char buf[200];

  int waiting_for_num;
  waiting_for_num= 0;

  int L;
  
  screen();
  waiting_for_num= 1;
  int op;
  op= 0;

  char *K;
  int digit;
  int mrc;
  mrc= 0;

  while(1)
  {
    int code;
    if (!gfx_event(NULL,&event)) continue;
    if (event.type==GFXEV_QUIT) break;
    if (event.type<GFXEV_LAST) continue;
    code= event.type - GFXEV_LAST;
    // Event is a button click

    if (code!='?') mrc= 0;

    switch(code)
    {
    case GFXEV_QUIT: return ;
    case '0': case '1': case '2': case '3': case '4':
    case '5': case '6': case '7': case '8': case '9': 
      if (waiting_for_num) { waiting_for_num= 0; killnum(); }
      if ((K=strchr(NUM2,':')))
      {
        digit= strlen(NUM2)-1 - (K-NUM2);
        if (digit==2) continue;
        if (digit==0 && code>='6') continue;
      }
      putnum(code);
      printnum();
      break;
    case 'G':
      code= '/';
    case ':':
    case '.':
      if (strhas(NUM2,":./")) continue;
      if (code=='/' && strchr(NUM2,'-')) continue;
      if (code=='/')
      {
        op= 0;
        gfx_label_set_text(GFX_UPDATE, lab2, "");
      }
      if (waiting_for_num) { waiting_for_num= 0; 
                             killnum(); putnum('0'); }
      putnum(code); 
      printnum();
      break;
    case 'N':
      if (strchr(NUM2,'/')) continue;
      if (waiting_for_num) continue;
      if (NUM2[0]=='-')
      {
        memmove(NUM2, NUM2+1, strlen(NUM2+1)+1);
      }
      else
      {
        int i;
        for(i=0;NUM2[i];i++) if (NUM2[i]>'0' && NUM2[i]<='9') break; 
        if (!NUM2[i]) continue;
        memmove(NUM2+1, NUM2, strlen(NUM2)+1);
        NUM2[0]= '-';
      }
      printnum();
      break;
    case '<':
      if (waiting_for_num) continue;
      L= strlen(NUM2);
      if (L==0) continue;
      NUM2[L-1]= 0;

      if (NUM2[0]=='-')
      {
        int i;
        for(i=0;NUM2[i];i++) if (NUM2[i]>'0' && NUM2[i]<='9') break; 
        if (!NUM2[i]) 
          memmove(NUM2, NUM2+1, strlen(NUM2+1)+1);
      }

      if (strlen(NUM2)==0)
      {
        NUM2[0]= '0';
        NUM2[1]= 0;
        waiting_for_num= 1;
      }
      printnum();
      break;
    case '/':
    case '*':
    case '+':
    case '-':
      if (strhas(NUM2,"/")) continue;
      if (op) do_op(op); 
      op= code;
      copy_number(code);
      waiting_for_num= 1;
      break;
    case '=':
      if (strhas(NUM2,"/"))
      {
        do_wind(); op= 0; waiting_for_num= 1; 
        break;
      }
      if (op) do_op(op);
      else waiting_for_num= 0;
      op= 0;
      break;
    case 'Y':
      if (waiting_for_num) continue;
      if (strhas(NUM2,":/-.")) continue;
      rwy= strtol(NUM2,NULL,10);
      rwy%= 360;
      sprintf(buf,"RWY: %03d", rwy);
      gfx_label_set_text(GFX_UPDATE,lab3, buf);
      NUM2[0]= '0'; NUM2[1]= 0;
      printnum();
      waiting_for_num= 1;
      break;
    case 'C':
      NUM2[0]= '0'; NUM2[1]= 0;
      NUM1[0]= 0;
      waiting_for_num= 1;
      gfx_label_set_text(GFX_UPDATE,lab1,NUM2);
      gfx_label_set_text(GFX_UPDATE,lab2,"");
      op= 0;
      break;
    case 'P':
    case 'M':
      if (strchr(NUM2,'/')) continue;
      if (MEM[0]==0) 
        strcpy(NUM1,"0");
      else
        strcpy(NUM1,MEM);
      if (code=='P') do_op('+'); else do_op('-');
      waiting_for_num= 1;
      strcpy(MEM,NUM2);
      gfx_label_set_text(GFX_UPDATE,lab4,"M");
      mrc= 1;
      break;
    case '?':
      if (MEM[0]==0) continue;
      if (mrc) { MEM[0]= 0; mrc= 0; gfx_label_set_text(GFX_UPDATE,lab4,""); } 
      else { strcpy(NUM2,MEM); printnum(); waiting_for_num= 0; mrc= 1; }
      waiting_for_num= 1;
      break;
    }
  }
}
